/*
Programm zur Erzeugung von Step-Impulsen für 3 Achsen zum Positionieren der Fräse (ohne Rechner)
    Copyright 2016  Harald Sattler

    Dieses Programm ist freie Software. Sie können es unter den Bedingungen der GNU General Public License, wie von der
    Free Software Foundation veröffentlicht, weitergeben und/oder modifizieren, entweder gemäß Version 3 der Lizenz
    oder (nach Ihrer Option) jeder späteren Version.

    Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE,
    sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
    Details finden Sie in der GNU General Public License.

    Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem Programm erhalten haben.
    Falls nicht, siehe <http://www.gnu.org/licenses/>.

Nützlich, um VOR Beginn eines Fräsvorgangs das Werkzeug zu wechseln,
den Hantierraum frei zu fahren (Spindel aus dem Weg) oder den Nullpunkt anzufahren.

Der G-Code-Streamer bekommt von dem Verfahren nichts mit, es ist _immer_ hinterher der Nullpunkt in der GUI zu setzen!

Achtung!
Serielle Schnittstelle ist auf 57600 Baud eingestellt. Beachten bei Verwendung der Menus (z.B. Shift-Ctrl-M in der Arduino-IDE)

Version 1, 30.12.2015, Harald Sattler
  Die Step- und Richtungs-Impulse werden nur erzeugt, wenn der Enable-Schalter eingeschaltet ist
  um zu verhindern, dass aus Versehen während eines Fräsvorgangs zusätzliche Schritte erzeugt werden.
  Das würde das Werkstück unweigerlich zerstören.

  Die Step-Impulse werden über Interrupts, die Dir-Signale per Programm erzeugt.
  Die beiden Daumen-Joysticks werden im Analog IRQ abgefragt.

  Einige Parameter für die Step-Erzeugung lassen sich per serielle Schnittstelle ändern und im EPROM des AtMega ablegen
  Die Parametrierung und die Abfrage der Werte erfolgt über USB mittels virtueller COM-SS
  Unterstützte Befehle: mit ?<RETURN> wird eine kurze Hilfe angezeigt

Version 2, 13.01.2016
- Die Enable-Signale für die Interfaceplatine müssen vom Arduino erzeugt werden, da offenbar die Verbindung der drei
  ENABLE-Eingänge an der Interface-Platine die Steuerung über den GRBL-Arduino verhindert -> anderes Layout des breakout board
- Die Richtungsinformation für pos. Joystickwerte wird im EEPROM gespeichert und über die serielle Com-SS parametrierbar gemacht

Version 3, 17.01.2016
- Umstellung auf Timer und Counter für die Step-Erzeugung. Timer1 (16bit) läuft mit 100kHz. Mit Zählern wird die Step-Frequenz erzeugt
- Die Ausgänge werden Tri-State geschaltet, wenn keine Step-Impulse erzeugt werden sollen -> GRBL-Eingänge aktiv

Version 4, 23.01.2016
- Umstellung auf Timer2 (8bit), die möglichen Zähler-Werte sind ohnehin kleiner als 255
  -> Vorteil: keine Vorkehrungen für atomare Zugriffe auf Timer-Variablen notwendig
- Werte aus einer Tabelle, exponentielle oder sigmoidale Funktion (jetzt exponentiell)
- ADC-Start in den Timer-IRQ verlegt -> keine Interferenzen (meistens), das klappt aber nur für eine Achse, Verwirbelungen bei 2..3 Achsen :(

Version 5, 25.01.2016
- Verwerfen des Analog-IRQ, Joysticks werden in der Loop ausgelesen
  -> keine Interferenzen mit Timer-IRQ
- Schleichgang für langsames Herantasten an Hindernisse, ausgelöst mit TASTER

Version 5.1, 25.01.2016
- Umstellung auf Timer1 (16bit) für bessere Auflösung der Stepabfolge und langsamere kleinste Verfahrgeschwindigkeit
    -> alle betroffenen Vars unsigned int statt byte
- Schleichgang wird daduch überflüssig -> TASTER disabled
- Menu: Defaults und Richtungsflags sind auskommentiert (Platzprobs bei global Vars)
- Defaults angepasst

Version 5.2, 27.01.2016
- Ersatz der beiden Speed-Tabellen durch Berechnung der Timer-Parameter zur Laufzeit (die Formel ist übersichtlich
    und die Berechnungszeit dürfte nicht stören (testen!!)

    mit speedX [1..(hugeness-1)]  (nicht ab 0 weil mit speedX == 0 keine Steps erzeugt werden)
    mit stepsize im setup() und nach Änderung von Startwert oder Endwert berechnet
    speedtempX = speedvalues[speedX];            wird zu...
      j = ((hugeness-1) - speedX);
      speedtempX = ((1/(hugeness - j)) * ((startwert + (j * stepsize)) + startwert));
      mit j substituiert...
      speedtempX = (((float)1/(hugeness-((hugeness-1) - speedX))) * (startwert + (((hugeness-1) - speedX) * stepsize)) + startwert);
- Durch Wegfall der Tabellen kann das Menu wieder vollständig angezeigt und erweitert werden

- Umstellung der Timer-Berechnung von float auf Festkomma (28.01.2016)
    speedtempX = (((10000 / (hugeness-((hugeness-1) - speedX))) / 10000) * (startwert + (((hugeness-1) - speedX) * stepsize)) + startwert);
    -> Durchlaufzeit der Berechnung je Achse von 9,2 ms auf 2,2 ms
- Anzahl diskreter Step-Intervalle (unterschiedliche Timer-Werte, = hugeness) einstellbar per Menu

- Version der Software in Hilfe-Seite anzeigen (03.02.2016)

Version 5.3, 25.02.2016
- Festkommazahlen-Mathematik ist (in meiner Implementierung) langsamer als Float, daher wieder Float
  (evtl. kann die Berechnung aufgeteilt werden, so dass die kleinen Werte über Festkomma berechnet werden, nur die Großen über Float)
- Ausgabe der Entfernung in mm auf ein Display über Seriell (vorerst mal auf SerialComInstruments)
- Reset der Zähler mit grüner Taste

04.03.2016
- Anzahl Werte (hugeness) auf unsigned int geändert um mit sehr großem Endwert sehr langsam fahren zu können und trotzdem gute Auflösung
  bei hohen Stepraten zu haben (hat nicht geklappt, daher wieder rückgängig gemacht)

03.05.2016
- Reset der Zähler auf Timer umgestellt

Version 5.4, 14.05.2016
- Hilfetext im ROM abgelegt (RAM einsparen)

29.05.2016
- Anzahl Werte (hugeness) und Zähler j für "$$" auf unsigned int geändert um mit sehr großem Endwert sehr langsam fahren zu können
  und trotzdem gute Auflösung bei hohen Stepraten zu haben
- Beschleunigungs- und Abbremsrampe bei X eingebaut (Prototyp, funktioniert nicht)

05.06.2016
- Beschleunigungs- und Abbremsrampe bei X, Y und Z eingebaut (funktioniert)
- Watchdog-IRQ für Serial-Problem bei SerialComInstruments (klappt nicht)

Version 5.5, 21.08.2016
- manuelles Verfahren zum Fräsen von geraden Kanten erfordert, dass nicht versehentlich die andere horizontale Achse bewegt wird.
  Vertikal kann weiterhin immer verfahren werden, die Blockade gilt nur für die horizontalen Achsen.
  Um das zu gewährleisten wird zu Beginn die grüne Taste am Controller gedrückt und dann die Achse per Joystick betätigt, die verwendet
  werden soll. Nach Erkennen der gewünschten Achse blinkt der grüne Taster und die andere horizontale Achse wird nicht mehr bewegt.
  Abschalten dieses Modus durch erneuten Druck auf den grünen Taster -> Blinken hört auf
  
27.06.2017  
- zusätzliches Flag um die Weg-Werte für SerialComInstruments abzuschalten
- Achtung! SCHALTER-Abfrage ist disabled - if(1==1)

Version 5.7, 05.11.2018
- Watchdog wieder entfernt, hat sowieso nichts gebracht
- Änderung der Step-Erzeugung um gleichzeitig alle Achsen verfahren zu können
- Joystick Potis mit AnalogScanner-Lib abfragen (Durchlaufzeit vorher ~350 ms, jetzt ~7 ms)

07.11.2018
- Offset in X-Richtung für die Kurve der Step-Geschwindigkeiten eingebaut (double Xoffset = 90;)
- Wert für Beschleunigung der Achsen parametrierbar gemacht (unsigned int maxAcc)
- EEPROM Startadresse für V5.7 auf 100 verlegt (damit die Parameter für V5.5 erhalten bleiben -> schneller Wechsel der Versionen)


mögliche Erweiterungen:
- Endschalter beim Verfahren abfragen um auf Block fahren zu verhindern. Bedingt aber mehrere Verbindungen zwischen JoyStep-Arduino
	und GRBL-Arduino -> unschöner Drahtverhau :(
- Probe abfragen (eher unsinnig, das kann SerialComCNC besser)
- temporären Nullpunkt setzen und per Button wieder anfahren (Übernahme der Kontrolle auf Joystick für Werkzeugwechsel)
    -> hier müssen Beschleunigungsrampen gefahren werden -> AccelStepper-Lib?
	-> lieber Anbindung des Joystick an GRBL mit Jog-Befehlen - siehe Joystep V6.1++
*/

#include <EEPROM.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>

#define ASCAN						// verwende die Lib AnalogScanner
#ifdef ASCAN
#include <AnalogScanner.h>			// Interrupt-driven analog input scanner class for Arduino
#endif // ASCAN


// defines
#define MajorVersion 5
#define MinorVersion 7

// Definition der Ausgänge (Arduino Nummerierung und AtMega Nummerierung)
#define dirX   4                        // Pin D4  = Port D4 = PortD Bit 4
#define stepX  2                        // Pin D2  = Port D2 = PortD Bit 2
#define dirY   7                        // Pin D7  = Port D7 = PortD Bit 7
#define stepY  5                        // Pin D5  = Port D5 = PortD Bit 5
#define dirZ  10                        // Pin D10 = Port B2 = PortB Bit 2
#define stepZ  8                        // Pin D8  = Port B0 = PortB Bit 0
#define LED   13                        // Pin D13 = Port B5 = PortB Bit 5
#define GREENLED 18                     // Pin D18/A4 ist Ausgang für Grüne LED im Taster

#define SCHALTER  19                    // Pin D19/A5 ist Eingang für den Enable-Schalter
#define TASTER  17                      // Pin D17/A3 ist Eingang für den Taster (bisher nicht verwendet)
#define JOYX A0							// Pin A0 ist Eingang für den Joystick der X-Achse
#define JOYY A1							// Pin A1 ist Eingang für den Joystick der Z-Achse
#define JOYZ A2							// Pin A2 ist Eingang für den Joystick der X-Achse

#define EnableX 12                      // drei getrennte Enable-Ausgänge für das Interface
#define EnableY 11
#define EnableZ 9

#define MIDDLE_INTERVAL 40              // Totraum um die Mittelstellung der Potis
// der Totraum koennte zur besseren Aufloesung kleiner ausfallen, das waere bei nur einer Achse auch kein Problem, eher gut, 
// aber da ich Kreuz-Sticks habe, muss ich verhindern, dass die jeweils andere Achse aus Versehen mitbewegt wird. 
// Dafuer muss der Totraum eher groesser gewaehlt werden.
// Das Intervall wandert ins EEPROM und wird per Menue aenderbar

// globale Variablen
unsigned int iMiddleInterval = MIDDLE_INTERVAL;	// Vorbelegung
String inputString = "";                // Speicher für Eingabestrings auf der seriellen SS
boolean stringComplete = false;         // CR/LF empfangen

int directionX;                         // Festlegung der Bewegungsrichtung der Stepper
int directionY;
int directionZ;
int eeAdr = 100;                        // Startadresse im EEPROM für Parameter
int debugprint = 0;                     // vorbelegen mit "keine Debugausgaben auf der seriellen SS"

int speedX;                             // aktuell mit dem Stick eingestelle Speed
int speedY;
int speedZ;
unsigned long tasterTime;				// Timer für Verzögertes Auslösen des Weg-Reset (Koordinaten auf Null setzen)
byte tasterLatch = 0;
unsigned long ledblinker = 0;

unsigned long lgetNewMidX = 0;			// Timer für Poti-Mittelstellung mitteln
unsigned long lgetNewMidY = 0;			// Timer für Poti-Mittelstellung mitteln
unsigned long lgetNewMidZ = 0;			// Timer für Poti-Mittelstellung mitteln

volatile unsigned int joystickX = 512;	// Mittelwerte vorbelegen für ersten Durchlauf
volatile unsigned int joystickY = 512;
volatile unsigned int joystickZ = 512;
unsigned int MittelJoyX;				// Mittenwerte der Achsenpotis ermitteln
unsigned int MittelJoyY;
unsigned int MittelJoyZ;
unsigned int MitteJoystickX;
unsigned int MitteJoystickY;
unsigned int MitteJoystickZ;

boolean lockX = false;                  // Flags für manuelles Verfahren, die jeweils andere Achse wird blockiert
boolean lockY = false;
boolean blinkGreen = false;             // manuelles Verfahren wird mit grüner LED blinkend signalisiert
boolean manuellMode = false;
byte wegvalues = 0;						// 0: keine Ausgaben fuer SerialComInstruments

unsigned int maxRateX;             		// Anzahl diskreter Step-Intervalle (unterschiedliche Timer-Werte)
unsigned int maxRateY;             		// Anzahl diskreter Step-Intervalle (unterschiedliche Timer-Werte)
unsigned int maxRateZ;             		// Anzahl diskreter Step-Intervalle (unterschiedliche Timer-Werte)
unsigned int maxRate;

volatile unsigned int speedtempX;       // vars werden in der IRQ-Routine verwendet -> volatile
volatile unsigned int speedtempY;
volatile unsigned int speedtempZ;
volatile unsigned int uscounterX = 0;
volatile unsigned int uscounterY = 0;
volatile unsigned int uscounterZ = 0;

byte i;
unsigned int startwert;                 // kleinster Wert für speedarray
unsigned int endwert;                   // größter Wert für speedarray
unsigned int hugeness;                  // Anzahl diskreter Step-Intervalle (unterschiedliche Timer-Werte)
unsigned int stepsize;                  //  = (endwert - startwert) / (hugeness-1);

unsigned int maxAcc = 800;
unsigned int maxincspeedStep = 800;   	// maximale Beschleunigung der Steps
unsigned int maxdecspeedStep = 800;   	// maximale Abbremsung der Steps
unsigned int sollspeedX;
unsigned int sollspeedY;
unsigned int sollspeedZ;
unsigned int oldspeedX;
unsigned int oldspeedY;
unsigned int oldspeedZ;

volatile long wegX = 0L;                // Zähler für Steps um zum Ausgangspunkt des Wegs zurück zu fahren (per AccelStepper Lib)
volatile long wegY = 0L;
volatile long wegZ = 0L;
long wegXsave = 0L;                     // Merker für Differenz
long wegYsave = 0L;
long wegZsave = 0L;
float wegXmm = 0L;                  	// Weg in mm
float wegYmm = 0L;
float wegZmm = 0L;
char wegBuffer[20];
volatile signed char Xdir = 0;
volatile signed char Ydir = 0;
volatile signed char Zdir = 0;

// AnalogScanner Instantiierung
#ifdef ASCAN
AnalogScanner scanner;
#endif

void setup() {
  for(i=0; i++; i<20) {wegBuffer[i] = 0;}
 // set pin modes
  pinMode(SCHALTER, INPUT_PULLUP);      // Eingang für Schalter (== Enable Joysticks)
                                        // ohne eingeschalteten Schalter (dessen LED leuchtet ROT) werden keine Steps erzeugt
  pinMode(TASTER, INPUT_PULLUP);

  pinMode(JOYX, INPUT);                 // Analogeingänge fuer Joysticks als Eingang definieren
  pinMode(JOYY, INPUT);
  pinMode(JOYZ, INPUT);

  pinMode(EnableX,INPUT);               // Signal EnableX zum Interface
  pinMode(EnableY,INPUT);               // Signal EnableY zum Interface
  pinMode(EnableZ,INPUT);               // Signal EnableZ zum Interface

  pinMode(LED,OUTPUT);                  // Ausgang LED
  pinMode(GREENLED,OUTPUT);             // Ausgang für Anzeige von Step-Aktivität (grüne LED im Schalter)
  //pinMode(13,OUTPUT);					// Ausgang für Messung mit Oszi // das ist der LED pin

  pinMode(dirX,OUTPUT);                 // Ausgang Dir X
  pinMode(dirY,OUTPUT);                 // Ausgang Dir Y
  pinMode(dirZ,OUTPUT);                 // Ausgang Dir Z
  pinMode(stepX,OUTPUT);                // Ausgang Step X
  pinMode(stepY,OUTPUT);                // Ausgang Step Y
  pinMode(stepZ,OUTPUT);                // Ausgang Step Z

  Serial.begin(57600);                  // fuer Debugging und Parametrierung
  inputString.reserve(20);              // 20 byte fuer inputString reservieren

  // Mittelstellung aller drei Achsen einlesen und merken, dann mehrere Samples nehmen und mitteln
  MittelJoyX = analogRead(JOYX);
  MittelJoyY = analogRead(JOYY);
  MittelJoyZ = analogRead(JOYZ);

  // Mittelwert von 20 Samples bilden
  for (i = 0; i < 20; i++)  {
	  MittelJoyX = ((MittelJoyX + analogRead(JOYX)) / 2);
	  Serial.print(MittelJoyX); Serial.print("; ");			// nur fuer Debug
	  MittelJoyY = ((MittelJoyY + analogRead(JOYY)) / 2);
	  // Serial.print(MittelJoyY); Serial.print("; ");
	  MittelJoyZ = ((MittelJoyZ + analogRead(JOYZ)) / 2);
	  // Serial.println(MittelJoyZ);
  }
  // ab hier keine Aufrufe an analog.Read() mehr, da jetzt der AnalogScanner läuft

  // AnalogScanner setup
  int scanOrder[] = { JOYX, JOYY, JOYZ };
  const int SCAN_COUNT = sizeof(scanOrder) / sizeof(scanOrder[0]); 
#ifdef ASCAN
  scanner.setScanOrder(SCAN_COUNT, scanOrder);
  //scanner.setAnalogReference(DEFAULT);

  scanner.beginScanning();				// start scanning der Joysticks
  delay(1); // Wait for the first scans to occur
#endif 

  eeAdr = 100;                            // Variablen aus dem EEPROM vorbelegen
  startwert = eepromReadInt(eeAdr);
  eeAdr += 2;
  endwert = eepromReadInt(eeAdr);
  eeAdr += 2;
  directionX = eepromReadInt(eeAdr);
  eeAdr += 2;
  directionY = eepromReadInt(eeAdr);
  eeAdr += 2;
  directionZ = eepromReadInt(eeAdr);
  eeAdr += 2;
  maxRateX = eepromReadInt(eeAdr);
  eeAdr += 2;
  maxRateY = eepromReadInt(eeAdr);
  eeAdr += 2;
  maxRateZ = eepromReadInt(eeAdr);
  eeAdr += 2;
  iMiddleInterval = eepromReadInt(eeAdr);
  eeAdr += 2;
  hugeness = eepromReadInt(eeAdr);
  eeAdr += 2;
  maxAcc = eepromReadInt(eeAdr);
  maxincspeedStep = maxdecspeedStep = maxAcc;

  for(i=0; i++; i<10) {
    digitalWrite(LED,1);
    delay(500);
    digitalWrite(LED,0);
    delay(500);
    }

// setup arrays for stepsize
calcStepsize();

cli();                                      // stop interrupts
{//setup timer1 (16 bit) --------------------------------------------------------------------------------------------
  TCCR1A = 0;                               // Timer/Counter1 Control Register A, set entire TCCR1A register to 0
  TCCR1B = 0;                               // Timer/Counter1 Control Register B, set entire TCCR1B register to 0
  TCCR1B |= (1 << WGM12);                   // turn on CTC mode
  // ----------------------
  //TCCR1B |= (1 << CS12);					// TMR Prescaler Select Bits  CS12   CS11   CS10              Timerrate
  TCCR1B |= (1 << CS11);					//                              1      1      1    -> external Clock source on T1. Rising Edge
  TCCR1B |= (1 << CS10);                    //                              1      1      0    -> external Clock source on T1. Falling Edge
                                            //                              1      0      1    -> 1024 ->  15,625 kHz
                                            //                              1      0      0    -> 256  ->  62,5 kHz
                                            //              --->>>          0      1      1    -> 64   ->  250 kHz
                                            //                              0      1      0    -> 8    ->  2 MHz
                                            //                              0      0      1    -> 1    ->  16 MHz
  // ----------------------
  TCCR1C = 0;                               // Timer/Counter1 Control Register C, set entire TCCR1C register to 0
  TCNT1  = 0;                               // initialize counter value to 0
  OCR1A = 5;								// set compare match register for xx khz increments
  //OCR1A = 5;								// ergibt ca. 11 kHz Aufruffrequenz der Timer-IRQ Routine
  // OCR1B =  timer1MaxVal;                             // set compare match register to 200 for 10 khz increments
  TIMSK1 |= (1 << OCIE1A);                  // enable timer1 compare interrupt A 
  // TIMSK1 |= (1 << OCIE1B);                  // enable timer1 compare interrupt B
}

sei();                                      // enable interrupts again
} // setup() end -----------------------------------------------------------------------------------------------------

ISR(TIMER1_COMPA_vect){                     //timer1 interrupt alle 10 µs, 100 kHz (prescaler 1, 159 TCC)
	//digitalWrite(LED, !digitalRead(LED));
   //PORTB |= (1 << PB5);                      // LED-Pin setzen
      
      if(speedtempX != 0)					// X-Achse
      {
        if(uscounterX++ > speedtempX)
        {
          PORTD |= (1 << PD2);              // Step-Pin setzen
        }
        if(uscounterX++ > speedtempX + 1)
        {
          PORTD &= ~(1 << PD2);             // Step-Pin löschen
          uscounterX = 0;
        }
      }
      
      if(speedtempY != 0)					// Y-Achse
      {
        if(uscounterY++ > speedtempY)
        {
          PORTD |= (1 << PD5);              // Step-Pin setzen
        }
        if(uscounterY++ > speedtempY + 1)
        {
          PORTD &= ~(1 << PD5);             // Step-Pin löschen
          uscounterY = 0;
        }
      }
      
      if(speedtempZ != 0)					// Z-Achse
      {
        if(uscounterZ++ > speedtempZ)
        {
          PORTB |= (1 << PB0);              // Step-Pin setzen
        }
        if(uscounterZ++ > speedtempZ + 1)
        {
          PORTB &= ~(1 << PB0);             // Step-Pin löschen
          uscounterZ = 0;
        }
      }
   //PORTB &= ~(1 << PB5);                     // LED-Pin loeschen
}

// Die Enables müssen Tristate geschaltet werden, damit der GRBL-Arduino Zugriff auf das Interface hat
boolean outputmode = false;                   // Merker, ob Outputs für Enable usw. aktiviert sind
unsigned long outputmodeTimer = 0;
int temp;                                     // Helfer für abs()-Wandlung, die hat plötzlich ohne nicht mehr funktioniert(??)

/****************************************************************************************************************************/
void loop()
/****************************************************************************************************************************/
{
	//Serial.println("Entered Loop");					// debug

  if(millis() > ledblinker + 1000) {
	//digitalWrite(LED, !digitalRead(LED));
  if(blinkGreen == true) { 	digitalWrite(GREENLED, !digitalRead(GREENLED));}
  if(blinkGreen == false) { 	digitalWrite(GREENLED, 0);}
	ledblinker = millis();
  }
  if(digitalRead(SCHALTER) == 0)              // hier den SCHALTER anschließen
//  if (1==1)
  {                                           // Steps nur ausgeben, wenn SCHALTER per SCHALTER auf LOW gezogen ist
    if(outputmode == false)                   // Richtung der Pins nur einmalig umschalten, wenn der Schalter betätigt wird, nicht bei jedem Durchlauf
    {
      outputmodeTimer = millis();
      // vorher die drei Enable-Ausgänge aktivieren um das Interface auf Joystick umzuschalten...
      pinMode(EnableX, OUTPUT);
      pinMode(EnableY, OUTPUT);
      pinMode(EnableZ, OUTPUT);
      digitalWrite(EnableX,0);					// die Enables sind active low
      digitalWrite(EnableY,0);
      digitalWrite(EnableZ,0);
      // ...und die Ausgangspins für Step und Dir auf OUTPUT schalten
      pinMode(stepX,OUTPUT);                    // enable step output (X-Achse)  // JA, NOTWENDIG !!!!
      pinMode(dirX,OUTPUT);                     // enable dir output (X-Achse)
      pinMode(stepY,OUTPUT);                    // enable step output (Y-Achse)
      pinMode(dirY,OUTPUT);                     // enable dir output (Y-Achse)
      pinMode(stepZ,OUTPUT);                    // enable step output (Z-Achse)
      pinMode(dirZ,OUTPUT);                     // enable dir output (Z-Achse)
      outputmode = true;
	  // digitalWrite(GREENLED,1);
    // digitalWrite(LED,1);
    }

    // ------------------------------------------------------------------------------------------------------------
    /// Alle drei Joysticks auf einen Rutsch auslesen
	//digitalWrite(LED, 1);
#ifdef ASCAN
	joystickX = scanner.getValue(JOYX);
	joystickY = scanner.getValue(JOYY);
	joystickZ = scanner.getValue(JOYZ);
#else
	joystickX = analogRead(JOYX);
	joystickY = analogRead(JOYY);   			// 512 nur fuer debug
	joystickZ = analogRead(JOYZ);  				// 512 nur fuer debug
#endif
	//digitalWrite(LED, 0);

	//Serial.println(joystickX);					// debug

	//joystickX = 512;	// nur fuer debug
	//joystickY = 512;	// nur fuer debug
	//joystickZ = 512;	// nur fuer debug

	double Xoffset = 90;			// verschiebt die speed Kurve im Koordinatensystem nach links -> bei geringer Auslenkung höhere TimerWerte

// ------------------------------------------------------------------------------------------------------------
    if(lockX == false)  {
		// PORTB |= (1 << PB5);							// LED-Pin setzen ,Laufzeit
		// joystickX: 43..946, in Ruhe: ~490
		// temp: in Ruhe 0
		temp = (joystickX - MittelJoyX);				// auf die Mitte skaliert
		//temp = -41;
      if(abs(temp) > iMiddleInterval)  {
		speedX = (int)map((long)joystickX, (long)(2*iMiddleInterval), (long)1023, (long)(-((long)maxRateX)), ((long)maxRateX));  // dann skalieren
		//Serial.print("joystickX "); Serial.print(joystickX);	Serial.print("    ");	// debug
		//Serial.print("(joystickX - MittelJoyX) "); Serial.print((int)(joystickX - MittelJoyX));	Serial.print("    ");	// debug
		//Serial.print("speedX "); Serial.print(speedX);	Serial.print("    ");	// debug

		digitalWrite(dirX, (((temp * directionX) < 0) ? 0 : 1));	// DIR pin setzen, abhängig von directionX

		speedX = constrain(abs(speedX), 0, maxRateX);
		// sollspeedX = ((1.0 / (double)(1.0 + (double)speedX + Xoffset)) * (double)((double)startwert + (double)((((double)maxRateX - 1.0) - ((double)speedX + Xoffset)) * (double)stepsize)) + (double)startwert);
		sollspeedX = ((1.0 / (1 + speedX + Xoffset)) * (startwert + (((maxRateX - 1) - (speedX + Xoffset)) * stepsize)) + startwert);
		//speedtempX = sollspeedX/10;
		//Serial.print("speedX "); Serial.print(speedX);	Serial.print("    ");	// debug
		//Serial.print("sollspeedX "); Serial.println(sollspeedX);		// debug

        if(sollspeedX < oldspeedX) {					// weniger Gas
          speedtempX -= map((long)speedtempX, (long)startwert, (long)endwert, (long)1, (long)maxincspeedStep);
          if(sollspeedX >= speedtempX) {
            oldspeedX = sollspeedX;						// letzte Stepgeschwindigkeit merken
          }
        }
        if(sollspeedX > oldspeedX) {					// mehr Gas

          speedtempX += map((long)speedtempX, (long)startwert, (long)endwert, (long)1, (long)maxdecspeedStep);

          if(sollspeedX <= speedtempX) {
            oldspeedX = sollspeedX;						// letzte Stepgeschwindigkeit merken
          }
        }
         //PORTB &= ~(1 << PB5);						// LED-Pin loeschen
      } else
      {													// Joystick ist in Mittelstellung -> keine Steps
        oldspeedX = endwert;							// Einstieg mit langsamster Stepgeschwindigkeit -> sehr großer Wert
        speedtempX = 0;									// Wert für Timer
        speedX = 0;
		if (millis() > lgetNewMidX + 1000)				// aber jede Sekunde den Mittelwert neu samplen
		{												// die Potis laufen über die Zeit weg!!
			lgetNewMidX = millis();
#ifdef ASCAN
			MittelJoyX = ((MittelJoyX + scanner.getValue(JOYX)) / 2);	// Mittelwert permanent anpassen
#else
			MittelJoyX = ((MittelJoyX + analogRead(JOYX)) / 2);			// Mittelwert permanent anpassen
#endif
			//Serial.print(joystickX); Serial.print(" ");
			//Serial.println(MittelJoyX);
		}
	  }
	//Serial.print("speedX "); Serial.println(speedX);	// debug
    }
    // ------------------------------------------------------------------------------------------------------------
    if(lockY == false)  {
		temp = (joystickY - MittelJoyY);				// auf die Mitte skaliert
		if (abs(temp) > iMiddleInterval) {
			speedY = (int)map((long)joystickY, (long)(2 * iMiddleInterval), (long)1023, (long)(-((long)maxRateY)), ((long)maxRateY));  // dann skalieren
			digitalWrite(dirY, (((temp * directionY) < 0) ? 0 : 1));	// DIR pin setzen, abhängig von directionX

			speedY = constrain(abs(speedY), 0, maxRateY);
			sollspeedY = ((1.0 / (double)(1.0 + (double)speedY + Xoffset)) * (double)((double)startwert + (double)((((double)maxRateY - 1.0) - ((double)speedY + Xoffset)) * (double)stepsize)) + (double)startwert);

			if (sollspeedY < oldspeedY) {				// weniger Gas
				speedtempY -= map((long)speedtempY, (long)startwert, (long)endwert, (long)1, (long)maxincspeedStep);
				if (sollspeedY >= speedtempY) {
					oldspeedY = sollspeedY;				// letzte Stepgeschwindigkeit merken
				}
			}
			if (sollspeedY > oldspeedY) {				// mehr Gas

				speedtempY += map((long)speedtempY, (long)startwert, (long)endwert, (long)1, (long)maxdecspeedStep);

				if (sollspeedY <= speedtempY) {
					oldspeedY = sollspeedY;				// letzte Stepgeschwindigkeit merken
				}
			}
		}
		else
		{												// Joystick ist in Mittelstellung -> keine Steps
			oldspeedY = endwert;						// Einstieg mit langsamster Stepgeschwindigkeit -> sehr großer Wert
			speedtempY = 0;								// Wert für Timer
			speedY = 0;
			if (millis() > lgetNewMidY + 1000)			// aber jede Sekunde den Mittelwert neu samplen
			{											// die Potis laufen über die Zeit weg!!
				lgetNewMidY = millis();
#ifdef ASCAN
				MittelJoyY = ((MittelJoyY + scanner.getValue(JOYY)) / 2);	// Mittelwert permanent anpassen
#else
				MittelJoyY = ((MittelJoyY + analogRead(JOYY)) / 2);			// Mittelwert permanent anpassen
#endif
			}
		}
	}
	// ------------------------------------------------------------------------------------------------------------
	temp = (joystickZ - MittelJoyZ);				// auf die Mitte skaliert
	if (abs(temp) > iMiddleInterval) {
		speedZ = (int)map((long)joystickZ, (long)(2 * iMiddleInterval), (long)1023, (long)(-((long)maxRateZ)), ((long)maxRateZ));  // dann skalieren
		digitalWrite(dirZ, (((temp * directionZ) < 0) ? 0 : 1));	// DIR pin setzen, abhängig von directionX

		speedZ = constrain(abs(speedZ), 0, maxRateZ);
		sollspeedZ = ((1.0 / (double)(1.0 + (double)speedZ + Xoffset)) * (double)((double)startwert + (double)((((double)maxRateZ - 1.0) - ((double)speedZ + Xoffset)) * (double)stepsize)) + (double)startwert);

		if (sollspeedZ < oldspeedZ) {				// weniger Gas
			speedtempZ -= map((long)speedtempZ, (long)startwert, (long)endwert, (long)1, (long)maxincspeedStep);
			if (sollspeedZ >= speedtempZ) {
				oldspeedZ = sollspeedZ;				// letzte Stepgeschwindigkeit merken
			}
		}
		if (sollspeedZ > oldspeedZ) {				// mehr Gas

			speedtempZ += map((long)speedtempZ, (long)startwert, (long)endwert, (long)1, (long)maxdecspeedStep);

			if (sollspeedZ <= speedtempZ) {
				oldspeedZ = sollspeedZ;				// letzte Stepgeschwindigkeit merken
			}
		}
	}
	else
	{												// Joystick ist in Mittelstellung -> keine Steps
		oldspeedZ = endwert;						// Einstieg mit langsamster Stepgeschwindigkeit -> sehr großer Wert
		speedtempZ = 0;								// Wert für Timer
		speedZ = 0;
		if (millis() > lgetNewMidZ + 1000)			// aber jede Sekunde den Mittelwert neu samplen
		{											// die Potis laufen über die Zeit weg!!
			lgetNewMidZ = millis();
#ifdef ASCAN
			MittelJoyZ = ((MittelJoyZ + scanner.getValue(JOYZ)) / 2);	// Mittelwert permanent anpassen
#else
			MittelJoyZ = ((MittelJoyZ + analogRead(JOYZ)) / 2);			// Mittelwert permanent anpassen
#endif
		}
	}
	// ------------------------------------------------------------------------------------------------------------
// Grüne LED im Taster der Joystick-Box wird eingeschaltet, solange Step-Impulse erzeugt werden
    // if ((speedX != 0) || (speedY != 0) || (speedZ != 0)) {digitalWrite(GREENLED,1);} else {digitalWrite(GREENLED,0);}
// ------------------------------------------------------------------------------------------------------------

// Bedienung einer Weg-Anzeige und deren Reset mit TASTER
    if(digitalRead(TASTER) == 0)  {
      if (tasterLatch == 0) {
        tasterTime = millis();
        tasterLatch = 1;
      }
      if(millis() > tasterTime + 3000 )  {  // Taster 3 sec gedrückt -> Reset Zähler
        wegX = 0;
        wegY = 0;
        wegZ = 0;
      }
      if((blinkGreen == true) && ((lockX == true) || (lockY == true)) && (manuellMode == false))  {     // manueller Modus ist aktiv und soll abgeschaltet werden
        blinkGreen = false;
        lockX = false;
        lockY = false;
        digitalWrite(GREENLED, 1);
      }
      if((speedX != 0) && (tasterLatch == 1)) {                     // X-Achse soll freigegeben, Y-Achse gesperrt werden
        blinkGreen = true;
        lockX = false;
        lockY = true;
        manuellMode = true;
      }
      if((speedY != 0) && (tasterLatch == 1)) {                     // Y-Achse soll freigegeben, X-Achse gesperrt werden
        blinkGreen = true;
        lockX = true;
        lockY = false;
        manuellMode = true;
      }
    }
    else {
        tasterLatch = 0;
        manuellMode = false;
        tasterTime = millis();
    }
    if((debugprint == 0) && (wegvalues == 1)) { // während Debug-Ausgaben keine Weg-Werte ausgeben
      if(wegXsave != wegX) {
        Serial.print("#41M");
        wegXmm = wegX * 0.001875;
        dtostrf(wegXmm,8,4,wegBuffer);
        Serial.print(wegBuffer);
        Serial.print('<');
        wegXsave = wegX;
      }
      // xxxx
      // wegY = 0;
      // wegZ = 0;
      if(wegYsave != wegY) {
        Serial.print("#42M");
        wegYmm = wegY * 0.001875;
        dtostrf(wegYmm,8,4,wegBuffer);
        Serial.print(wegBuffer);
        Serial.print('<');
        wegYsave = wegY;
      }
      if(wegZsave != wegZ) {
        Serial.print("#43M");
        wegZmm = wegZ * 0.001875;
        dtostrf(wegZmm,8,4,wegBuffer);
        Serial.print(wegBuffer);
        Serial.print('<');
        wegZsave = wegZ;
      }
    }
    if(debugprint == 1) {         // debugprint kann über die serielle SS gesetzt werden
      Serial.print(joystickX); Serial.print("\t"); Serial.print(speedX); Serial.print("\t"); Serial.print(speedtempX); Serial.print("\t");
      Serial.print(joystickY); Serial.print("\t"); Serial.print(speedY); Serial.print("\t"); Serial.print(speedtempY); Serial.print("\t");
      Serial.print(joystickZ); Serial.print("\t"); Serial.print(speedZ); Serial.print("\t"); Serial.println(speedtempZ);
    }
  }                               // Schalter EIN hier zu Ende
  else
  {                               // Schalter AUS
    // Richtung der Pins nur einmalig umschalten, wenn der Schalter betätigt wird, nicht bei jedem Durchlauf
	if((outputmode == true) && ((outputmodeTimer + 500) < millis()))	{  // 500 millisec gegen Prellen beim Einschalten
      // die drei Enable-Ausgänge wieder deaktivieren um das Interface auf GRBL umzuschalten, wenn der SCHALTER-Schalter aus ist...
      // ...und die Ausgangspins auf INPUT (= HiZ) schalten
      pinMode(stepX,INPUT);                    	// disable step und dir output (X-Achse)
      pinMode(dirX,INPUT);
      pinMode(stepY,INPUT);                    	// disable step und dir output (Y-Achse)
      pinMode(dirY,INPUT);
      pinMode(stepZ,INPUT);                    	// disable step und dir output (Z-Achse)
      pinMode(dirZ,INPUT);
      digitalWrite(EnableX,1);                  // Enables deaktivieren (sind low activ)
      digitalWrite(EnableY,1);
      digitalWrite(EnableZ,1);
      pinMode(EnableX, INPUT);                  // Enable-Pins wieder in HiZ, damit der GRBL-Arduino Zugriff auf das Interface bekommt
      pinMode(EnableY, INPUT);
      pinMode(EnableZ, INPUT);
      outputmode = false;
      // digitalWrite(GREENLED,0);
	}
// ------------------------------------------------------------------------------------------------------------
  }

// Parameter verändern oder im EEPROM speichern oder dort holen
  if (stringComplete == true) {
    Serial.println(inputString);                                    // Echo eingegebenen String

	if ((inputString.indexOf(F("?")) >= 0) || (inputString.indexOf(F("help")) >= 0))   // Hilfe ausgeben 
	{
		Serial.print(F("Version ")); Serial.print(MajorVersion); Serial.print("."); Serial.print(MinorVersion); Serial.print(", "); Serial.print(__DATE__); Serial.println(F(", Harald Sattler"));
		Serial.println(F("\nHilfe"));											// \n -> eine Leerzeile oberhalb "Hilfe"
		Serial.println(F("Folgende Befehle werden verstanden (Gross/Kleinschreibung ist zu beachten!):"));
		Serial.println(F("? / help       -> diese Hilfe"));
		Serial.println(F("$$             -> listet Parameter"));
        Serial.println(F("startwert=xx -> neuer unterer Wert (schnelles Stepping) fuer SpeedTable (*)"));
        Serial.println(F("endwert=xxx  -> neuer oberer Wert (langsames Stepping) fuer SpeedTable (*)"));
        Serial.println(F("stepint=xxx  -> neue Anzahl diskreter StepIntervalle (*)"));
		Serial.println(F("dirX=x         -> Laufrichtung bei positiven Joystick X Werten. x=[-1|1] (*)"));
		Serial.println(F("dirY=x         -> Laufrichtung bei positiven Joystick Y Werten. x=[-1|1] (*)"));
		Serial.println(F("dirZ=x         -> Laufrichtung bei positiven Joystick Z Werten. x=[-1|1] (*)"));
		Serial.println(F("maxRateX=x     -> maximale Step-Rate der X-Achse. x=[10..2500] (*)"));
		Serial.println(F("maxRateY=x     -> maximale Step-Rate der Y-Achse. x=[10..2500] (*)"));
		Serial.println(F("maxRateZ=x     -> maximale Step-Rate der Z-Achse. x=[10..2500] (*)"));
		Serial.println(F("midInt=x       -> ab welcher Auslenkung werden Steps erzeugt. x=[30..60] (*)"));
		Serial.println(F("maxAcc=x       -> Beschleunigung/Abbremsung. x=[500..1600] (*)"));
		Serial.println(F("save         -> Parameter mit(*) werden ins EEPROM gesichert"));
        Serial.println(F("recall       -> Parameter mit(*) werden aus dem EEPROM geladen"));
        Serial.println(F("defaults     -> Parameter werden mit Defaults vorbelegt"));
        Serial.println(F("start        -> Debugausgaben starten"));
        Serial.println(F("stop         -> Debugausgaben anhalten\n"));
	}
    if(inputString.indexOf(F("$$")) >= 0) {                        // vorhandene Werte abfragen/ausgeben
        Serial.print(F("StepDir X: ")); Serial.print(directionX); Serial.print(F("\tStepDir Y: ")); Serial.print(directionY); Serial.print(F("\tStepDir Z: ")); Serial.println(directionZ);
        Serial.print(F("StartWert: ")); Serial.print(startwert); Serial.print(F("\tEndWert: ")); Serial.println(endwert);
		Serial.print(F("maxRateX: ")); Serial.print(maxRateX);
		Serial.print(F(" maxRateY: ")); Serial.print(maxRateY);
		Serial.print(F(" maxRateZ: ")); Serial.println(maxRateZ);
		Serial.print(F("maxAcc: ")); Serial.println(maxAcc);
		Serial.print(F("MiddleInterval: ")); Serial.println(iMiddleInterval);
		Serial.print(F("StepIntervalle: ")); Serial.print(hugeness); Serial.print(F("\tStepSize: ")); Serial.println(stepsize);
        unsigned int ttt = ((1.0 / (1+ iMiddleInterval)) * (startwert + (((hugeness-1) - (iMiddleInterval)) * stepsize)) + startwert);
        Serial.print(F("SpeedValue @MIDDLE_INTERVAL (")); Serial.print(iMiddleInterval); Serial.print("): "); Serial.println(ttt);
        Serial.print(F("SpeedValues: "));
        for(unsigned int j=0; j<hugeness; j++) {
              // int ttt = (((float)1/(hugeness-((hugeness-1) - i))) * (startwert + (((hugeness-1) - i) * stepsize)) + startwert);
              // int ttt = (((10000 / (hugeness-((hugeness-1) - i))) / 10000) * (startwert + (((hugeness-1) - i) * stepsize)) + startwert);
              // int ttt = (((10000L/(1+i)) * (startwert+(((hugeness-1)-i)*stepsize)))/10000)+startwert;
              ttt = ((1.0 / (1+j)) * (startwert + (((hugeness-1) - (j)) * stepsize)) + startwert);
              Serial.print(ttt); Serial.print(",");}
        Serial.println("");
        }
    if(inputString.indexOf(F("startwert")) >= 0) {                // StartWert des Stepsize-Array ändern
        int equalSign = inputString.indexOf('=');
        inputString.remove(0,equalSign+1);
        startwert = inputString.toInt();
        Serial.print(F("StartWert: ")); Serial.print(startwert); Serial.print("\tendwert: "); Serial.println(endwert);
        calcStepsize();                                                // setup arrays for stepsize
      }
    if(inputString.indexOf(F("endwert")) >= 0) {                  // EndWert des Stepsize-Array ändern
        int equalSign = inputString.indexOf('=');
        inputString.remove(0,equalSign+1);
        endwert = inputString.toInt();
        Serial.print(F("StartWert: ")); Serial.print(startwert); Serial.print("\tEndWert: "); Serial.println(endwert);
        calcStepsize();                                                // setup arrays for stepsize
      }
	if (inputString.indexOf(F("maxRateX")) >= 0) {                  // xxx des Stepsize-Array ändern
		int equalSign = inputString.indexOf('=');
		inputString.remove(0, equalSign + 1);
		maxRateX = inputString.toInt();
		Serial.print(F("max StepRateX: ")); Serial.println(maxRateX);
	}
	if (inputString.indexOf(F("maxRateY")) >= 0) {                  // xxx des Stepsize-Array ändern
		int equalSign = inputString.indexOf('=');
		inputString.remove(0, equalSign + 1);
		maxRateY = inputString.toInt();
		Serial.print(F("max StepRateY: ")); Serial.println(maxRateY);
	}
	if (inputString.indexOf(F("maxRateZ")) >= 0) {                  // xxx des Stepsize-Array ändern
		int equalSign = inputString.indexOf('=');
		inputString.remove(0, equalSign + 1);
		maxRateZ = inputString.toInt();
		Serial.print(F("max StepRateZ: ")); Serial.println(maxRateZ);
	}
	if (inputString.indexOf(F("maxAcc")) >= 0) {                  // xxx des Stepsize-Array ändern
		int equalSign = inputString.indexOf('=');
		inputString.remove(0, equalSign + 1);
		maxAcc = inputString.toInt();
		Serial.print(F("max Acceleration: ")); Serial.println(maxAcc);
		maxincspeedStep = maxdecspeedStep = maxAcc;
	}
	if (inputString.indexOf(F("dirX")) >= 0) {                     // directionX ändern
		int equalSign = inputString.indexOf('=');
		inputString.remove(0, equalSign + 1);
		directionX = inputString.toInt();
		Serial.print(F("X Richtung "));
		if (directionX == 1) Serial.print(F("positiv"));
		else if (directionX == -1) Serial.print(F("negativ"));
		else Serial.print(F("undefiniert!"));
	}
	if (inputString.indexOf(F("dirY")) >= 0) {                     // directionY ändern
		int equalSign = inputString.indexOf('=');
		inputString.remove(0, equalSign + 1);
		directionY = inputString.toInt();
		Serial.print(F("Y Richtung "));
		if (directionY == 1) Serial.print(F("positiv"));
		else if (directionY == -1) Serial.print(F("negativ"));
		else Serial.print(F("undefiniert!"));
	}
	if (inputString.indexOf(F("dirZ")) >= 0) {                     // directionZ ändern
		int equalSign = inputString.indexOf('=');
		inputString.remove(0, equalSign + 1);
		directionZ = inputString.toInt();
		Serial.print(F("Z Richtung "));
		if (directionZ == 1) Serial.print(F("positiv"));
		else if (directionZ == -1) Serial.print(F("negativ"));
		else Serial.print(F("undefiniert!"));
	}
	if (inputString.indexOf(F("midInt")) >= 0) {                   // Joystick Totraum ändern
		int equalSign = inputString.indexOf('=');
		inputString.remove(0, equalSign + 1);
		iMiddleInterval = inputString.toInt();
		Serial.print(F("MiddleInterval: ")); Serial.println(iMiddleInterval);
		Serial.println(F(""));
	}
	if(inputString.indexOf(F("stepint")) >=0) {                  // EndWert des Stepsize-Array ändern
        int equalSign = inputString.indexOf('=');
        inputString.remove(0,equalSign+1);
        hugeness = inputString.toInt();
        Serial.print(F("StepIntervalle: ")); Serial.println(hugeness);
        calcStepsize();                                                // setup arrays for stepsize
      }
    if(inputString.indexOf(F("save")) >= 0) {                     // Parameter ins EEPROM schreiben
        Serial.println(F("Params -> EEPROM"));
        eeAdr = 100;
        eepromWriteInt(eeAdr, startwert);
        eeAdr += 2;
        eepromWriteInt(eeAdr, endwert);
        eeAdr += 2;
        eepromWriteInt(eeAdr, directionX);
        eeAdr += 2;
        eepromWriteInt(eeAdr, directionY);
        eeAdr += 2;
        eepromWriteInt(eeAdr, directionZ);
		eeAdr += 2;
		eepromWriteInt(eeAdr, maxRateX);
		eeAdr += 2;
		eepromWriteInt(eeAdr, maxRateY);
		eeAdr += 2;
		eepromWriteInt(eeAdr, maxRateZ);
		eeAdr += 2;
		eepromWriteInt(eeAdr, iMiddleInterval);
		eeAdr += 2;
        eepromWriteInt(eeAdr, hugeness);
		eeAdr += 2;
        eepromWriteInt(eeAdr, maxAcc);
      }
    if(inputString.substring(0, 6) == F("recall")) {                   // Parameter aus EEPROM laden
        Serial.println(F("Params aus EEPROM laden"));
        eeAdr = 100;
        startwert = eepromReadInt(eeAdr);
        eeAdr += 2;
        endwert = eepromReadInt(eeAdr);
        eeAdr += 2;
        directionX = eepromReadInt(eeAdr);
        eeAdr += 2;
        directionY = eepromReadInt(eeAdr);
        eeAdr += 2;
        directionZ = eepromReadInt(eeAdr);
		eeAdr += 2;
		maxRateX = eepromReadInt(eeAdr);
		eeAdr += 2;
		maxRateY = eepromReadInt(eeAdr);
		eeAdr += 2;
		maxRateZ = eepromReadInt(eeAdr);
		eeAdr += 2;
		iMiddleInterval = eepromReadInt(eeAdr);
		eeAdr += 2;
        hugeness = eepromReadInt(eeAdr);
		eeAdr += 2;
		maxAcc = eepromReadInt(eeAdr);

		Serial.print(F("StepDir X: ")); Serial.print(directionX); Serial.print(F("\tStepDir Y: ")); Serial.print(directionY); Serial.print(F("\tStepDir Z: ")); Serial.println(directionZ);
        Serial.print(F("Startwert: ")); Serial.print(startwert); Serial.print(F("\tEndwert: ")); Serial.println(endwert);
        Serial.print(F("Diskrete StepIntervalle: ")); Serial.print(hugeness); Serial.print(F("\tStepsize: ")); Serial.println(stepsize);
		Serial.print(F("maxRateX: ")); Serial.print(maxRateX); Serial.print(F(" maxRateY: ")); Serial.print(maxRateY); Serial.print(F(" maxRateZ: ")); Serial.println(maxRateZ);
		Serial.print(F("maxAcc: ")); Serial.println(maxAcc);
		Serial.print(F("MiddleInterval: ")); Serial.println(iMiddleInterval);
		Serial.println(F(""));
      }
    if(inputString.substring(0, 8) == F("defaults")) {                 // Default Parameter laden
        Serial.println(F("Default Parameter laden"));
        startwert = 14;                           // unterer Grenzwert für timer
        endwert = 18000;                          // oberer Grenzwert für timer
        directionX = 1;                           // Step direction positiv bei positiven Joystickwerten
        directionY = 1;
        directionZ = 1;
		maxRateX = maxRateY = 2000;
		maxRateZ = 600;
		iMiddleInterval = MIDDLE_INTERVAL;
		hugeness = 255;
		maxAcc = 800;
		maxincspeedStep = maxdecspeedStep = maxAcc;
        calcStepsize();                           // setup arrays for stepsize
        Serial.print(F("StartWert: ")); Serial.print(startwert); Serial.print(F("\tEndWert: ")); Serial.print(endwert); Serial.print(F("\tStepsize: ")); Serial.println(stepsize);
        Serial.print(F("StepDir X: ")); Serial.print(directionX); Serial.print(F("\tStepDir Y: ")); Serial.print(directionY); Serial.print(F("\tStepDir Z: ")); Serial.println(directionZ);
		Serial.print(F("maxRateX: ")); Serial.print(maxRateX); Serial.print(F(" maxRateY: ")); Serial.print(maxRateY); Serial.print(F(" maxRateZ: ")); Serial.println(maxRateZ);
		Serial.print(F("maxAcc: ")); Serial.println(maxAcc);
		Serial.print(F("MiddleInterval: ")); Serial.println(iMiddleInterval);
		Serial.println(F(""));
	}
    if(inputString.substring(0, 5) == F("start")) {                        // debug Werte auf Seriell ausgeben
        Serial.print(F("Debug print started"));
        Serial.println("");
        debugprint = 1;
        }
    if(inputString.substring(0, 4) == F("stop")) {                        // debug Ausgaben stoppen
        Serial.print(F("Debug print stopped"));
        debugprint = 0;
        }
    // clear the string
    inputString = "";
    stringComplete = false;
  }
} // loop end
// ------------------------------------------------------------------------------------------------
 // Abfrage der seriellen Schnittstelle
void serialEvent() {
    //digitalWrite(GREENLED,1);
  while (Serial.available()) {
    // get the new byte:
    char inChar = (char)Serial.read();
    // add it to the inputString:
    inputString += inChar;
    // if the incoming character is a newline, set a flag
    // so the main loop can do something about it:
    if (inChar == '\n') {
      stringComplete = true;
    }
  }
    // digitalWrite(GREENLED,0);
}
// ------------------------------------------------------------------------------------------------
 // Hilfsroutinen Integer in/aus EEPROM
void eepromWriteInt(int adr, int wert) {
byte low, high;
  low=wert&0xFF;
  high=(wert>>8)&0xFF;
  EEPROM.write(adr, low); // dauert 3,3ms
  EEPROM.write(adr+1, high);
  return;
} //eepromWriteInt

int eepromReadInt(int adr) {
byte low, high;
  low=EEPROM.read(adr);
  high=EEPROM.read(adr+1);
  return low + ((high << 8)&0xFF00);
} //eepromReadInt

// ------------------------------------------------------------------------------------------------
// Hilfsroutine Stepsize-Arrays berechnen
void calcStepsize() {
   stepsize = ((endwert - startwert) / (hugeness-1));
} //calcStepsize

